#ifndef FILETIME_COUNTER_HPP
#define FILETIME_COUNTER_HPP

#include <windows.h>

namespace hrt {

struct filetime_counter
{
	typedef unsigned __int64 value_type;

	static value_type frequency()
	{
		return static_cast<value_type>(FREQUENCY);
	}

	static value_type value()
	{
		value_type value;
		::GetSystemTimeAsFileTime(reinterpret_cast<LPFILETIME>(&value));
		return value;
	}

	static bool is_tickincrement(const value_type& value)
	{
		return (tickincrement() == value);
	}

	static value_type tickincrement()
	{
		DWORD	dwTimeAdjustment = 0,
					dwTimeIncrement = 0,
          dwClockTick = 0;
		BOOL	fAdjustmentDisabled = TRUE;

		::GetSystemTimeAdjustment(&dwTimeAdjustment, &dwTimeIncrement, &fAdjustmentDisabled);
		dwClockTick = (FALSE == fAdjustmentDisabled) ? dwTimeAdjustment : dwTimeIncrement;
		
		return static_cast<value_type>(dwClockTick);
	}

private:

	enum
	{
		FREQUENCY = 10000000 // 100 nanosecond period
	};
};

} // namespace hrt

#endif // FILETIME_COUNTER_HPP