#ifndef REFERENCE_POINT_HPP
#define REFERENCE_POINT_HPP

#include <hrt/filetime_counter.hpp>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

namespace hrt {

namespace internal {

//
// reference_point: data struct containing various time information
//									for a certain point in time
//
template<typename counter_type>
struct reference_point
{
	typedef counter_type													counter_type;
	typedef typename counter_type::value_type			counter_integral_type;
	typedef typename filetime_counter::value_type	filetime_integral_type;

	//
	// CONSTRUCTORS
	//
	reference_point(const counter_integral_type& counter_value = 0,
									const filetime_integral_type& filetime_value = 0,
									const counter_integral_type& accuracy = 0,
									const filetime_integral_type& tickincrement = 0)
		: m_counter_value(counter_value),
			m_filetime_value(filetime_value),
			m_accuracy(accuracy),
			m_tickincrement(tickincrement)
	{}

	reference_point(const reference_point& other)
	{
		assign(other);
	}

	//
	// OPERATOR OVERLOADING
	//
	reference_point& operator=(const reference_point& other)
	{ 
		if (&other != this)
		{
      return assign(other);
		}

		return *this;
	}

	//
	// ACCESSORS
	//
	counter_integral_type	counter() const
	{ return m_counter_value; }

	filetime_integral_type filetime() const
	{ return m_filetime_value; }

	counter_integral_type	accuracy() const
	{ return m_accuracy; }

	filetime_integral_type tickincrement() const
	{ return m_tickincrement; }

	//
	// MANIPULATORS
	//
	void set_counter(const counter_integral_type& counter_value)
	{ m_counter_value = counter_value; }

	void set_filetime(const filetime_integral_type& filetime_value)
	{ m_filetime_value = filetime_value; }

	void set_accuracy(const counter_integral_type& accuracy)
	{ m_accuracy = accuracy; }

	void set_tickincrement(const filetime_integral_type& tickincrement)
	{ m_tickincrement = tickincrement; }

private:
  //
	// HELPERS
	//
	reference_point& assign(const reference_point& other)
	{
    ::CopyMemory(this, &other, sizeof(*this));
		return *this;
	}

private:
	//
	// DATA
	//
	counter_integral_type		m_counter_value;
	filetime_integral_type	m_filetime_value;
	counter_integral_type		m_accuracy;
	filetime_integral_type	m_tickincrement;
};

} // namespace internal

} // namespace hrt

#endif // REFERENCE_POINT_HPP
