#ifndef SCOPED_THREAD_AFFINITY_HPP
#define SCOPED_THREAD_AFFINITY_HPP

#define WIN32_MEAN_AND_LEAN
#include <windows.h>
#include <stdexcept>

namespace hrt {

namespace internal {

class scoped_thread_affinity
{
	scoped_thread_affinity(const scoped_thread_affinity&);						// not defined
	scoped_thread_affinity& operator=(const scoped_thread_affinity&); // not defined

public:
	//
	// CONSTRUCTORS
	//
	scoped_thread_affinity(DWORD_PTR affinity_mask)
		: m_dismissed(false),
			m_old_affinity_mask(::SetThreadAffinityMask(GetCurrentThread(), affinity_mask))
	{
		if (0 == m_old_affinity_mask)
		{
			throw std::runtime_error("Failed to set thread affinity mask");
		}
	}

	~scoped_thread_affinity()
	{
		dismiss();
	}

	void dismiss()
	{
		if (!m_dismissed)
		{
			::SetThreadAffinityMask(::GetCurrentThread(), m_old_affinity_mask);
			m_dismissed = true;
		}
	}

private:
	//
	// IMPLEMENTATION
	//
	bool			m_dismissed;
	DWORD_PTR	m_old_affinity_mask;
};

} // namespace internal

} // namespace hrt

#endif // SCOPED_THREAD_AFFINITY_HPP
