#ifndef SCOPED_THREAD_PRIORITY_HPP
#define SCOPED_THREAD_PRIORITY_HPP

#include <windows.h>

namespace hrt {

namespace internal {

class scoped_thread_priority
{
	scoped_thread_priority(const scoped_thread_priority&);						// not defined
	scoped_thread_priority& operator=(const scoped_thread_priority&); // not defined

public:
	//
	// CONSTRUCTORS
	//
	scoped_thread_priority(int thread_priority)
		: m_dismissed(false),
			m_old_thread_priority(::GetThreadPriority(::GetCurrentThread()))
	{
		::SetThreadPriority(::GetCurrentThread(), thread_priority);
	}

	~scoped_thread_priority()
	{
		dismiss();
	}

	void dismiss()
	{
		if (!m_dismissed)
		{
			::SetThreadPriority(::GetCurrentThread(), m_old_thread_priority);
			m_dismissed = true;
		}
	}

	private:
		//
		// IMPLEMENTATION
		//
		bool	m_dismissed;
		int		m_old_thread_priority;
};

} // namespace internal

} // namespace hrt

#endif // SCOPED_THREAD_PRIORITY_HPP
