#ifndef THREAD_PRIORITY_HPP
#define THREAD_PRIORITY_HPP

#include <hrt/internal/scoped_thread_priority.hpp>
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

namespace hrt {

namespace internal {

//
// thread_priority: Simple class to facilitate incrementation of the current thread's
//									priority. Restores original priority on destruction.
//
class thread_priority
{
public:
	//
	// CONSTRUCTORS
	//
	thread_priority()
		: m_scoped_priority(current_priority()),
			m_current(current_priority())
	{}

public:
	//
	// MANIPULATORS
	//
	int operator++(int)
	{
		int previous_priority = m_current;
		increase_priority();
		return previous_priority;
	}

private:
	//
	// STATIC HELPERS
	//
	static int current_priority()
	{ return ::GetThreadPriority(::GetCurrentThread()); }

private:
	//
	// HELPERS
	//
	void increase_priority()
	{
		static const int basic_priorities[] = 
		{
			THREAD_PRIORITY_IDLE,
			THREAD_PRIORITY_LOWEST,
			THREAD_PRIORITY_BELOW_NORMAL,
			THREAD_PRIORITY_NORMAL,
			THREAD_PRIORITY_ABOVE_NORMAL,
			THREAD_PRIORITY_HIGHEST,
			THREAD_PRIORITY_TIME_CRITICAL
		};

		for (int i = 0;
				 i < sizeof(basic_priorities)/sizeof(basic_priorities[0]);
				 i++)
		{
			if (basic_priorities[i] > m_current)
			{
				set_priority(basic_priorities[i]);
				break;
			}
		}
	}

	void set_priority(int new_priority)
	{
		if (0 == ::SetThreadPriority(::GetCurrentThread(), new_priority))
		{
			::OutputDebugStringA("Warning: failed to set thread priority");
		}
		else
		{
			m_current = current_priority();
		}
	}

private:
	//
	// IMPLEMENTATION
	//
	scoped_thread_priority	m_scoped_priority;
	int											m_current;
};

} // namespace internal

} // namespace hrt

#endif // THREAD_PRIORITY_HPP
