#ifndef TIME_DATA_HPP
#define TIME_DATA_HPP

#include <hrt/filetime_counter.hpp>
#include <hrt/internal/reference_point.hpp>

namespace hrt {

namespace internal {

template<typename counter_type>
struct time_data
{
public:
	//
	// TYPES
	//
	typedef reference_point<counter_type>	reference_point_type;

public:
	//
	// CONSTRUCTORS
	//
	time_data()
		: m_frequency(0.0),
			m_relative_frequency(0.0)
		{}

	//
	// ACCESSORS
	//
	double frequency() const
		{ return m_frequency; }
	double relative_frequency() const
		{ return m_relative_frequency; }
	const reference_point_type& ref() const
		{ return m_ref; }

	//
	// MANIPULATORS
	//
	void set_frequency(const double& frequency)
	{
		m_frequency = frequency;
		m_relative_frequency = calculate_relative_frequency(frequency);
	}

	void set_ref(const reference_point_type& ref)
		{ m_ref = ref; }

private:
	//
	// STATIC HELPERS
	//
	static double calculate_relative_frequency(const double& frequency)
	{
		return frequency/static_cast<double>(filetime_counter::frequency());
	}

private:
	//
	// DATA
	//
	reference_point_type	m_ref;
	double								m_frequency;
	double								m_relative_frequency;
};

} // namespace internal

} // namespace hrt

#endif TIME_DATA_HPP

