#ifndef PERFORMANCE_COUNTER_HPP
#define PERFORMANCE_COUNTER_HPP

#include <hrt/counter_traits.hpp>
#include <windows.h>
#include <string>

namespace hrt {

struct performance_counter
{
	typedef __int64 value_type;

	static value_type frequency()
	{
		value_type frequency;
		::QueryPerformanceFrequency(reinterpret_cast<LARGE_INTEGER*>(&frequency));
		return frequency;
	}

	static value_type value()
	{
		value_type value;
		::QueryPerformanceCounter(reinterpret_cast<LARGE_INTEGER*>(&value));
		return value;
	}
};

//
// counter_traits specializations
//
template<>
struct counter_traits<performance_counter>
{
	static std::string description()
	{ return "performance counter"; }
	static bool needs_initial_adjustment()
	{ return true; }
};

} // namespace hrt

#endif // PERFORMANCE_COUNTER_HPP