#ifndef TIME_STAMP_HPP
#define TIME_STAMP_HPP

#include <hrt/time_span.hpp>

namespace hrt {

//
// Class time_stamp: represents a high-resolution time stamp
//
template<typename counter_type>
class time_stamp
{
	typedef time_span<counter_type>						time_span_type;
	typedef typename counter_type::value_type	value_type;

public:

	//
	// CONSTRUCTORS
	//
	time_stamp(const value_type& stamp)
		: m_stamp(stamp) {}

	//
	// CONSTRUCTORS
	//
	time_stamp()
		: m_stamp(0)
		{}
	time_stamp(const time_stamp& other)
		: m_stamp(other.m_stamp)
		{}

	//
	// MANIPULATORS
	//
	time_stamp& operator=(const time_stamp& other)
		{ if (this != &other) { m_stamp = other.m_stamp; } return *this; }

	//
	// ACCESSORS
	//
	value_type value() const
		{ return m_stamp; }

	//
	// FRIENDS
	//
	friend
	time_span<counter_type> operator-(const time_stamp<counter_type>& first, const time_stamp<counter_type>& second)
	{
		return time_span<counter_type>(first.m_stamp-second.m_stamp);
	}

private:
	//
	// IMPLEMENTATION
	//
	value_type m_stamp;
};

//template<typename counter_type>
//inline time_span<counter_type> operator-(const time_stamp<counter_type>& first, const time_stamp<counter_type>& second)
//{
//	return time_span<counter_type>(first.m_stamp-second.m_stamp);
//}

} // namespace hrt

#endif // TIME_STAMP_H
