#ifndef SAMPLE_COMMON_H
#define SAMPLE_COMMON_H

//
// Avoid min/max macro definitions
//
#if defined (min) || defined (max)
#	error Include this file before any other file, or define NOMINMAX globally
#else
#	define NOMINMAX
#endif

//
// We need NT 4.0 or better
//
#ifndef _WIN32_WINNT
# define _WIN32_WINNT 0x0400
#endif

//
// Common hrt includes 
//
#include <hrt/performance_counter.hpp>
#include <hrt/system_time.hpp>
#include <hrt/file_time.hpp>
#include <hrt/time_provider.hpp>
#include <hrt/timer.hpp>
#include <hrt/time_stamp.hpp>
#include <hrt/time_span.hpp>
#include <hrt/tsc_counter.hpp>
#include <ostream>
#include <iomanip>

//
// Application-wide typedefs
//
typedef hrt::performance_counter							counter_t;
typedef hrt::time_provider<counter_t, 50>			time_provider_t;

//
// Operator overloading
//
inline
std::basic_ostream<char>&
operator<<(std::basic_ostream<char>& ostr, const hrt::system_time& systime)
{
	char prevFill = ostr.fill('0');

	ostr << std::setw(4) << systime.year() << '-'
			 << std::setw(2) << systime.month() << '-'
			 << std::setw(2) << systime.day() << ' '
       << std::setw(2) << systime.hour() << ':'
			 << std::setw(2) << systime.minute() << ':'
			 << std::setw(2) << systime.second() << '.'
			 << std::setw(3) << systime.millis();

	ostr.fill(prevFill);

	return ostr;
}

inline
std::basic_ostream<char>&
operator<<(std::basic_ostream<char>& ostr, const hrt::file_time& filetime)
{
	return operator<<(ostr, hrt::system_time(filetime));
}

#endif // SAMPLE_COMMON_H
