#ifndef LOGGINGIMPL_H
#define LOGGINGIMPL_H

#include "LoggingThread.h"
#include "SimpleEvent.h"
#include <windows.h>
#include <cassert>
#include <string>
#include <vector>

//
// LoggingImpl: basic implementation of ILogging interface
//
template<typename TimeProvider>
class LoggingImpl : public ILogging
{
public:
	//
	// TYPES
	//
	typedef TimeProvider								TimeProviderType;
	typedef SimpleEvent<TimeProvider>		EventType;
	typedef std::vector<EventType>			EventVector;
	typedef typename EventType::RawTime	RawTimeType;

public:
	//
	// STRUCTORS
	//
	LoggingImpl(TimeProviderType* pProvider)
		: m_pProvider(pProvider)
	{
		assert(0 != m_pProvider);
		::InitializeCriticalSection(&m_csEvents);
	}

	~LoggingImpl()
	{
		::DeleteCriticalSection(&m_csEvents);
	}

public:
	//
	// ILogging implementation
	//
	virtual void LogEvent(const std::string& sLogEvent)
	{
		RawTimeType rtTimeStamp = m_pProvider->rawtime();
		::EnterCriticalSection(&m_csEvents);
		m_vEvents.push_back(EventType(rtTimeStamp, ::GetCurrentThreadId(), sLogEvent));
		::LeaveCriticalSection(&m_csEvents);
	}

public:
	//
	// MANIPULATORS
	//
	void GetEvents(EventVector& vEvents)
	{
		::EnterCriticalSection(&m_csEvents);
		vEvents = m_vEvents;
		m_vEvents.clear();
		::LeaveCriticalSection(&m_csEvents);
	}

private:
	//
	// IMPLEMENTATION
	//
	TimeProvider*			m_pProvider;
	EventVector				m_vEvents;
	CRITICAL_SECTION	m_csEvents;
};

#endif // LOGGINGIMPL_H
