#include "stdafx.h"
#include "LoggingThread.h"
#include <memory>

namespace {

struct INTERNAL_DATA
{
	INTERNAL_DATA(ILogging* pLogging_, HANDLE hSignalEvent_, HANDLE hQuitEvent_)
		: pLogging(pLogging_),
			hSignalEvent(hSignalEvent_),
			hQuitEvent(hQuitEvent_)
	{}

	ILogging* pLogging;
	HANDLE		hSignalEvent;
	HANDLE		hQuitEvent;
};

typedef INTERNAL_DATA*	PINTERNAL_DATA;

DWORD WINAPI LoggingThread(void* pvThreadData)
{
	assert(0 != pvThreadData);

	std::auto_ptr<INTERNAL_DATA>	pInternalData(static_cast<PINTERNAL_DATA>(pvThreadData));
	ILogging*											pLogging = pInternalData->pLogging;
	HANDLE												arWaitObjects[2] = { pInternalData->hSignalEvent, pInternalData->hQuitEvent };
	DWORD													dwWaitResult = 0;

	pLogging->LogEvent("Starting up");
	pLogging->LogEvent("Waiting for signal event");

	//
	// Loop until quit event received (found guilty of assigning and comparing in the same statement)
	//
	while (WAIT_OBJECT_0+1 != (dwWaitResult = ::WaitForMultipleObjects(sizeof(arWaitObjects)/sizeof(arWaitObjects[0]),
																																		 &arWaitObjects[0],
																																		 FALSE,
																																		 INFINITE)))
	{
		assert(WAIT_OBJECT_0 == dwWaitResult);
		pLogging->LogEvent("Signal event received, waiting for next");
	}

	pLogging->LogEvent("Quit event received");

	return ::GetLastError();
}

} // end anonymous namespace

HANDLE CreateLoggingThread(ILogging* pLogging, HANDLE hSignalEvent, HANDLE hQuitEvent)
{
	assert(0 != pLogging);

	PINTERNAL_DATA	pInternal = new INTERNAL_DATA(pLogging, hSignalEvent, hQuitEvent);
	DWORD						dwUnusedThreadId = 0;

	return ::CreateThread(NULL,
												0,
                        &LoggingThread,
                        static_cast<void*>(pInternal),
                        0,
                        &dwUnusedThreadId);
}
