#ifndef OUTPUTLOGEVENT_H
#define OUTPUTLOGEVENT_H

#include "SimpleEvent.h"
#include <hrt/system_time.hpp>
#include <windows.h>
#include <cassert>
#include <cmath>
#include <functional>
#include <iomanip>
#include <ostream>

//
// OutputLogEvents: simple functor used for formatting & outputting log events
//
template<typename TimeProviderType>
struct OutputLogEvent : public std::unary_function<SimpleEvent<TimeProviderType>, void>
{
	OutputLogEvent(std::basic_ostream<char>* pOstream,
								 TimeProviderType* pProvider,
								 int nOutputTimePrecision)
		: m_pOstream(pOstream),
			m_pProvider(pProvider),
			m_nOutputTimePrecision(nOutputTimePrecision)
	{
		assert(0 != m_pOstream);
	}

	void operator()(const SimpleEvent<TimeProviderType>& event)
	{
		FILETIME			ftTimeStamp;
		LARGE_INTEGER liTimeStamp;

		m_pProvider->filetime_from_rawtime(event.m_rt, &ftTimeStamp);

		liTimeStamp.LowPart = ftTimeStamp.dwLowDateTime;
		liTimeStamp.HighPart = ftTimeStamp.dwHighDateTime;

		double seconds = static_cast<double>(liTimeStamp.QuadPart)/FILETIME_FREQUENCY;
		seconds = std::fmod(seconds, 60.0);

		hrt::system_time	stTimeStamp(&ftTimeStamp);

		char chOldFill = m_pOstream->fill('0');

		(*m_pOstream) << std::dec << std::setw(2)
										<< stTimeStamp.hour() << ':'
									<< std::setw(2)
										<< stTimeStamp.minute() << ':'
									<< std::setw(m_nOutputTimePrecision + 3) << std::fixed << std::setprecision(m_nOutputTimePrecision)
										<< seconds << ' '
									<< "Thread:" << std::hex << std::setw(8)
										<< event.m_dwOriginatingThreadId << ' '
                  << event.m_sMsg
										<< '\n';

		m_pOstream->fill(chOldFill);
	}

private:
	enum { FILETIME_FREQUENCY = 10000000 };
	std::basic_ostream<char>* m_pOstream;
	TimeProviderType* m_pProvider;
	int m_nOutputTimePrecision;
};

#endif // OUTPUTLOGEVENT_H
